const { genNamedExport, genImport } = require("./utils");

module.exports = (
  file,
  api,
  { from, default: defaultImport, importList: _importList }
) => {
  const j = api.jscodeshift;
  const importList = _importList ?? [];
  const root = j(file.source);

  // inject import

  let isPluginInstalled = false;

  if (defaultImport) {
    const defaultList = root.find(j.ImportDeclaration, (node) => {
      if (node.specifiers) {
        let rst = false;
        node.specifiers.forEach(item => {
          if (
            j.ImportDefaultSpecifier.check(item)
            && item.local && item.local.name === defaultImport
          ) {
            rst = true;
            return;
          }
          if (
            j.ImportSpecifier.check(item)
            && importList.includes(item.local.name)
          ) {
            const importIdx = importList.indexOf(item.local.name);
            if (importIdx > -1) {
              importList.splice(importIdx, 1);
            }
          }
          return;
        });
        return rst;
      }
    });

    isPluginInstalled = defaultList.nodes().length > 0;
  }

  if (!isPluginInstalled || importList.length) {
    const newImport = genImport(j, from,
      isPluginInstalled ? null : defaultImport
      , ...importList);


    root.nodes()[0].program.body.unshift(newImport);
  }

  if (!isPluginInstalled) {
    const installNode = root.find(j.FunctionDeclaration, (node) => {
      if (node.id.name === "install") {
        return true;
      }
    });

    // inject install
    const newExpress = j.expressionStatement(
      j.callExpression(
        j.memberExpression(j.identifier("app"), j.identifier("use")),
        [j.identifier(defaultImport)]
      )
    );
    installNode.nodes()[0].body.body.push(newExpress);
  }

  // inject export
  const exportNode = root.find(j.ExportNamedDeclaration, (node) => {
    return true;
  });

  if (importList.length) {
    exportNode.nodes()[0].specifiers.push(
      ...importList.map((namedImport) => {
        return genNamedExport(j, namedImport);
      })
    );
  }

  return root.toSource();
};
