const { genImport } = require("./utils");

function injectImport(
  file,
  api,
  { from, default: defaultImport, importList: _importList }
) {
  const j = api.jscodeshift;
  const importList = _importList ?? [];
  const root = j(file.source);

  let isPluginInstalled = false;

  if (defaultImport) {
    const defaultList = root.find(j.ImportDeclaration, (node) => {
      if (node.specifiers) {
        let rst = false;
        node.specifiers.forEach(item => {
          if (
            j.ImportDefaultSpecifier.check(item)
            && item.local && item.local.name === defaultImport
          ) {
            rst = true;
            return;
          }
          if (
            j.ImportSpecifier.check(item)
            && importList.includes(item.local.name)
          ) {
            const importIdx = importList.indexOf(item.local.name);
            if (importIdx > -1) {
              importList.splice(importIdx, 1);
            }
          }
          return;
        });
        return rst;
      }
    });

    isPluginInstalled = defaultList.nodes().length > 0;
  }

  if (!isPluginInstalled || importList.length) {
    const newImport = genImport(j, from,
      isPluginInstalled ? null : defaultImport
      , ...importList);


    root.nodes()[0].program.body.unshift(newImport);
  }
  return root.toSource();
}

function injectFunctionUse(file, api, { fnName, usedName }) {
  const j = api.jscodeshift;
  const root = j(file.source);

  const renderNode = root.find(j.FunctionDeclaration, (node) => {
    if (node.id.name === fnName) {
      return true;
    }
  });

  let isPluginInstalled = false;

  if (usedName) {
    const defaultList = root.find(j.CallExpression, (node) => {
      if (
        node.arguments
        && node.arguments.length
        && j.Identifier.check(node.arguments[0])
        && node.arguments[0].name === usedName
      ) {
        return true;
      }
      return false;
    });

    isPluginInstalled = defaultList.nodes().length > 0;
  }


  if (!isPluginInstalled) {
    const newExpress = j.expressionStatement(
      j.callExpression(
        j.memberExpression(j.identifier("app"), j.identifier("use")),
        [j.identifier(usedName)]
      )
    );
    renderNode.nodes()[0].body.body.splice(2, 0, newExpress);
  }

  return root.toSource();
}

module.exports = {
  injectImport,
  injectFunctionUse,
};
